@@ -5,12 +5,6 @@ import re
 import zipfile
 import pkg_resources
 import tempfile
-import urlparse
-import urllib2
-import urllib
-import ConfigParser
-from distutils.sysconfig import get_python_version
-from email.FeedParser import FeedParser
 from pip.locations import bin_py, running_under_virtualenv
 from pip.exceptions import InstallationError, UninstallationError
 from pip.vcs import vcs
@@ -21,7 +15,10 @@ from pip.util import is_installable_dir, is_local, dist_is_local
 from pip.util import renames, normalize_path, egg_link_path
 from pip.util import make_path_relative
 from pip import call_subprocess
-from pip.backwardcompat import any, copytree
+from pip.backwardcompat import (any, copytree, urlparse, urllib,
+                                ConfigParser, string_types, HTTPError,
+                                FeedParser, get_python_version,
+                                b)
 from pip.index import Link
 from pip.locations import build_prefix
 from pip.download import (get_file_content, is_url, url_to_path,
@@ -37,7 +34,7 @@ class InstallRequirement(object):
 
     def __init__(self, req, comes_from, source_dir=None, editable=False,
                  url=None, update=True):
-        if isinstance(req, basestring):
+        if isinstance(req, string_types):
             req = pkg_resources.Requirement.parse(req)
         self.req = req
         self.comes_from = comes_from
@@ -110,7 +107,7 @@ class InstallRequirement(object):
         if self.satisfied_by is not None:
             s += ' in %s' % display_path(self.satisfied_by.location)
         if self.comes_from:
-            if isinstance(self.comes_from, basestring):
+            if isinstance(self.comes_from, string_types):
                 comes_from = self.comes_from
             else:
                 comes_from = self.comes_from.from_path()
@@ -123,7 +120,7 @@ class InstallRequirement(object):
             return None
         s = str(self.req)
         if self.comes_from:
-            if isinstance(self.comes_from, basestring):
+            if isinstance(self.comes_from, string_types):
                 comes_from = self.comes_from
             else:
                 comes_from = self.comes_from.from_path()
@@ -220,7 +217,8 @@ class InstallRequirement(object):
         finally:
             logger.indent -= 2
         if not self.req:
-            self.req = pkg_resources.Requirement.parse(self.pkg_info()['Name'])
+            self.req = pkg_resources.Requirement.parse(
+                "%(Name)s==%(Version)s" % self.pkg_info())
             self.correct_build_location()
 
     ## FIXME: This is a lame hack, entirely for PasteScript which has
@@ -238,7 +236,7 @@ def replacement_run(self):
             writer(self, ep.name, egg_info.os.path.join(self.egg_info,ep.name))
     self.find_sources()
 egg_info.egg_info.run = replacement_run
-execfile(__file__)
+exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))
 """
 
     def egg_info_data(self, filename):
@@ -406,14 +404,24 @@ execfile(__file__)
 
         pip_egg_info_path = os.path.join(dist.location,
                                          dist.egg_name()) + '.egg-info'
+        # workaround for http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=618367
+        debian_egg_info_path = pip_egg_info_path.replace(
+            '-py%s' % pkg_resources.PY_MAJOR, '')
         easy_install_egg = dist.egg_name() + '.egg'
         develop_egg_link = egg_link_path(dist)
-        if os.path.exists(pip_egg_info_path):
+
+        pip_egg_info_exists = os.path.exists(pip_egg_info_path)
+        debian_egg_info_exists = os.path.exists(debian_egg_info_path)
+        if pip_egg_info_exists or debian_egg_info_exists:
             # package installed by pip
-            paths_to_remove.add(pip_egg_info_path)
+            if pip_egg_info_exists:
+                egg_info_path = pip_egg_info_path
+            else:
+                egg_info_path = debian_egg_info_path
+            paths_to_remove.add(egg_info_path)
             if dist.has_metadata('installed-files.txt'):
                 for installed_file in dist.get_metadata('installed-files.txt').splitlines():
-                    path = os.path.normpath(os.path.join(pip_egg_info_path, installed_file))
+                    path = os.path.normpath(os.path.join(egg_info_path, installed_file))
                     paths_to_remove.add(path)
             if dist.has_metadata('top_level.txt'):
                 if dist.has_metadata('namespace_packages.txt'):
@@ -510,7 +518,7 @@ execfile(__file__)
                     dirname = os.path.join(dirpath, dirname)
                     name = self._clean_zip_name(dirname, dir)
                     zipdir = zipfile.ZipInfo(self.name + '/' + name + '/')
-                    zipdir.external_attr = 0755 << 16L
+                    zipdir.external_attr = 0x1ED << 16 # 0o755
                     zip.writestr(zipdir, '')
                 for filename in filenames:
                     if filename == PIP_DELETE_MARKER_FILENAME:
@@ -536,11 +544,10 @@ execfile(__file__)
         temp_location = tempfile.mkdtemp('-record', 'pip-')
         record_filename = os.path.join(temp_location, 'install-record.txt')
         try:
-
             install_args = [
                 sys.executable, '-c',
                 "import setuptools;__file__=%r;"\
-                "execfile(__file__)" % self.setup_py] +\
+                "exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))" % self.setup_py] +\
                 list(global_options) + [
                 'install',
                 '--single-version-externally-managed',
@@ -610,7 +617,7 @@ execfile(__file__)
             ## FIXME: should we do --install-headers here too?
             call_subprocess(
                 [sys.executable, '-c',
-                 "import setuptools; __file__=%r; execfile(%r)" % (self.setup_py, self.setup_py)]
+                 "import setuptools; __file__=%r; exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))" % self.setup_py]
                 + list(global_options) + ['develop', '--no-deps'] + list(install_options),
 
                 cwd=self.source_dir, filter_stdout=self._filter_install,
@@ -728,6 +735,37 @@ deleted (unless you remove this file).
 '''
 
 
+class Requirements(object):
+
+    def __init__(self):
+        self._keys = []
+        self._dict = {}
+
+    def keys(self):
+        return self._keys
+
+    def values(self):
+        values_list = []
+        for key in self._keys:
+            values_list.append(self._dict[key])
+        return values_list
+
+    def __contains__(self, item):
+        return item in self._keys
+    
+    def __setitem__(self, key, value):
+        if key not in self._keys:
+            self._keys.append(key)
+        self._dict[key] = value
+    
+    def __getitem__(self, key):
+        return self._dict[key]
+
+    def __repr__(self):
+        values = [ '%s: %s' % (repr(k), repr(self[k])) for k in self.keys() ]
+        return 'Requirements({%s})' % ', '.join(values)
+
+
 class RequirementSet(object):
 
     def __init__(self, build_dir, src_dir, download_dir, download_cache=None,
@@ -739,7 +777,7 @@ class RequirementSet(object):
         self.download_cache = download_cache
         self.upgrade = upgrade
         self.ignore_installed = ignore_installed
-        self.requirements = {}
+        self.requirements = Requirements()
         # Mapping of alias: real_name
         self.requirement_aliases = {}
         self.unnamed_requirements = []
@@ -776,7 +814,7 @@ class RequirementSet(object):
 
     @property
     def has_requirements(self):
-        return self.requirements.values() or self.unnamed_requirements
+        return list(self.requirements.values()) or self.unnamed_requirements
 
     @property
     def has_editables(self):
@@ -816,7 +854,7 @@ class RequirementSet(object):
         ## FIXME: duplicates code from install_files; relevant code should
         ##        probably be factored out into a separate method
         unnamed = list(self.unnamed_requirements)
-        reqs = self.requirements.values()
+        reqs = list(self.requirements.values())
         while reqs or unnamed:
             if unnamed:
                 req_to_install = unnamed.pop(0)
@@ -852,7 +890,7 @@ class RequirementSet(object):
     def prepare_files(self, finder, force_root_egg_info=False, bundle=False):
         """Prepare process. Create temp directories, download and/or unpack files."""
         unnamed = list(self.unnamed_requirements)
-        reqs = self.requirements.values()
+        reqs = list(self.requirements.values())
         while reqs or unnamed:
             if unnamed:
                 req_to_install = unnamed.pop(0)
@@ -915,7 +953,8 @@ class RequirementSet(object):
                         if url:
                             try:
                                 self.unpack_url(url, location, self.is_download)
-                            except urllib2.HTTPError, e:
+                            except HTTPError:
+                                e = sys.exc_info()[1]
                                 logger.fatal('Could not install requirement %s because of error %s'
                                              % (req_to_install, e))
                                 raise InstallationError(
@@ -948,6 +987,15 @@ class RequirementSet(object):
                             if bundle and req_to_install.url:
                                 self.copy_to_build_dir(req_to_install)
                                 install = False
+                        # req_to_install.req is only avail after unpack for URL pkgs
+                        # repeat check_if_exists to uninstall-on-upgrade (#14)
+                        req_to_install.check_if_exists()
+                        if req_to_install.satisfied_by:
+                            if self.upgrade:
+                                req_to_install.conflicts_with = req_to_install.satisfied_by
+                                req_to_install.satisfied_by = None
+                            else:
+                                install = False
                 if not is_bundle and not self.is_download:
                     ## FIXME: shouldn't be globally added:
                     finder.add_dependency_links(req_to_install.dependency_links)
@@ -956,7 +1004,8 @@ class RequirementSet(object):
                         for req in req_to_install.requirements():
                             try:
                                 name = pkg_resources.Requirement.parse(req).project_name
-                            except ValueError, e:
+                            except ValueError:
+                                e = sys.exc_info()[1]
                                 ## FIXME: proper warning
                                 logger.error('Invalid requirement: %r (%s) in requirement %s' % (req, e, req_to_install))
                                 continue
@@ -1024,9 +1073,9 @@ class RequirementSet(object):
 
     def install(self, install_options, global_options=()):
         """Install everything in this set (after having downloaded and unpacked the packages)"""
-        to_install = sorted([r for r in self.requirements.values()
-                             if self.upgrade or not r.satisfied_by],
-                            key=lambda p: p.name.lower())
+        to_install = [r for r in self.requirements.values()
+                      if self.upgrade or not r.satisfied_by]
+
         if to_install:
             logger.notify('Installing collected packages: %s' % (', '.join([req.name for req in to_install])))
         logger.indent += 2
@@ -1114,16 +1163,12 @@ class RequirementSet(object):
 
     def bundle_requirements(self):
         parts = [self.BUNDLE_HEADER]
-        for req in sorted(
-            [req for req in self.requirements.values()
-             if not req.comes_from],
-            key=lambda x: x.name):
+        for req in [req for req in self.requirements.values()
+                    if not req.comes_from]:
             parts.append('%s==%s\n' % (req.name, req.installed_version))
         parts.append('# These packages were installed to satisfy the above requirements:\n')
-        for req in sorted(
-            [req for req in self.requirements.values()
-             if req.comes_from],
-            key=lambda x: x.name):
+        for req in [req for req in self.requirements.values()
+                    if req.comes_from]:
             parts.append('%s==%s\n' % (req.name, req.installed_version))
         ## FIXME: should we do something with self.unnamed_requirements?
         return ''.join(parts)
@@ -1249,7 +1294,7 @@ def parse_editable(editable_req, default_vcs=None):
     else:
         req = match.group(1)
     ## FIXME: use package_to_requirement?
-    match = re.search(r'^(.*?)(?:-dev|-\d.*)', req)
+    match = re.search(r'^(.*?)(?:-dev|-\d.*)$', req)
     if match:
         # Strip off -dev, -0.2, etc.
         req = match.group(1)
@@ -1392,19 +1437,21 @@ class UninstallPthEntries(object):
 
     def remove(self):
         logger.info('Removing pth entries from %s:' % self.file)
-        fh = open(self.file, 'r')
+        fh = open(self.file, 'rb')
+        # windows uses '\r\n' with py3k, but uses '\n' with py2.x
         lines = fh.readlines()
         self._saved_lines = lines
         fh.close()
-        try:
-            for entry in self.entries:
-                logger.info('Removing entry: %s' % entry)
+        if any(b('\r\n') in line for line in lines):
+            endline = '\r\n'
+        else:
+            endline = '\n'
+        for entry in self.entries:
             try:
-                lines.remove(entry + '\n')
+                logger.info('Removing entry: %s' % entry)
+                lines.remove(b(entry + endline))
             except ValueError:
                 pass
-        finally:
-            pass
         fh = open(self.file, 'wb')
         fh.writelines(lines)
         fh.close()
@@ -1428,6 +1475,12 @@ class FakeFile(object):
 
     def readline(self):
         try:
-            return self._gen.next()
+            try:
+                return next(self._gen)
+            except NameError:
+                return self._gen.next()
         except StopIteration:
             return ''
+
+    def __iter__(self):
+        return self._gen
